==> ocre_v0_031.tgz: ocre-0.031/ocre/src/imagio.c <==

#include <libintl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define _(mens) gettext(mens)
#define N_(mens) (mens)

/*
#include <strings.h>
*/

#include "config.h"
#include "errors.h"
#include "imag.h"

enum {tfimagDesconocida, tfimagPbm, tfimagPgm};

FILE * mifopen (char *nf, char *modo)
{
  FILE * fd;
  char nfl[1024];
  fd = fopen (nf, modo);
  if (fd != 0L)
    return fd;
  // strcpy  (nfl, "../../ocre-d");
  strcpy  (nfl, "dat");
  strncat (nfl, "/",2);
  strncat (nfl, nf, 128);
  fd = fopen (nfl, modo);
  return fd;
}

int tipofImagen (char * nf)
{
  FILE * fd;
  char  buf[MAXCOM] ;
  fd = mifopen (nf, "r");
  compruebaFd (fd, nf, "imagio.tipofImagen");
  fgets (buf, MAXCOM, fd);
  fclose (fd);
/**/
  if (strcmp(buf, "P4\n")==0)
    return tfimagPbm ;
  if (strcmp(buf, "P5\n")==0)
    return tfimagPgm ;
  return tfimagDesconocida;
}

void leeCabeceraImagenPgbm (FILE *fd, timagf imagf, int tfima)
{
  char  buf[MAXCOM] ;
  char * vcom[MAXNLCOM];
  int n, nc;
  int tx, ty;
  int ng;
  fgets (buf, MAXCOM, fd);
      /* asert P5 */
  nc = 0;
  do
    {
    fgets (buf, MAXCOM, fd);
    vcom [nc] = strdup (buf);
    nc ++;
    }
  while (buf[0] == '#');
  nc --;
  free (vcom[nc]);
  while (buf[0] == '\n')
    fgets (buf, MAXCOM, fd);
  imagf->comen = (char **) malloc (nc * sizeof(char *));
  for(n=0; n<nc; n++)
    imagf->comen[n] = vcom[n];
  imagf->ncom = nc;
  sscanf (buf, "%d %d", &tx, &ty);
  if (tx > MAXPIXH) {
    printf (_("%s: x_dimension (%4d) larger than anticipated (%4d)\n"),
              "ocre.imagio.leeCabeceraImagenPgbm", tx, MAXPIXH);
    exit (9);
    }
  if (ty > MAXPIXV) {
    printf (_("%s: y_dimension (%4d) larger than anticipated (%4d)\n"),
              "ocre.imagio.leeCabeceraImagenPgbm", ty, MAXPIXV);
    exit (9);
    }
  imagf->tx   = tx;
  imagf->ty   = ty;
  if (tfima == tfimagPgm) {
    fscanf (fd, "%d", &ng);
        /* asert ng == 255 */
    fread (buf, 1, 1, fd);   /* lee el cambio de linea */
    }
}

void leePixelsPbm (char *donde, int tx, int ty, FILE * fd)
{
  int x, y;
  char c;
  int m = 128;   /* '= 128' para no tener avisos(warnings) */
  char *d;
  d = donde;
  for (y=0; y<ty; y++){
    for (x=0; x<tx; x++){
      if ((x%8)==0) {
        fread (&c, 1, 1, fd);
        if (c <0) c = c + 256;
        m = 128;
        }
      if (c & m)
        *d = 32;
      else
        *d = 224;
      m /= 2;
      d++ ;
      }
    }
}

void leeImagenPgbm (timagf * nimagf, char * nf, int tfima)
{
  FILE * fd;
  timagf imagf;
  int tam;
  fd = mifopen (nf, "r");
  compruebaFd (fd, nf, "imagio.leeImagenPgm");
  imagf = imagfalloc ();
  leeCabeceraImagenPgbm (fd, imagf, tfima);
  tam  = imagf->tx * imagf->ty;
  imagf->apixels = (char *) malloc ((size_t)tam) ;
      /* if mal malloc */
  if (tfima == tfimagPgm) {
    fread (imagf->apixels, 1, tam, fd);
        /* if mal fread */
    }
  else /* tfima == tfimaPbm */ {
    leePixelsPbm (imagf->apixels, imagf->tx, imagf->ty, fd);
    }
  fclose (fd);
  imagf->nf = strdup (nf);
      /* if traza ... */
  imagf->ven0 = ventBaseDeImagf (imagf);
  * nimagf = imagf;
}

void leeImagen (timagf * nimagf, char * nf)
{
  switch (tipofImagen(nf)) {
    case tfimagPbm :
      leeImagenPgbm (nimagf, nf, tfimagPbm);
      break ;
    case tfimagPgm :
      leeImagenPgbm (nimagf, nf, tfimagPgm);
      break ;
    case tfimagDesconocida :
      printf ("ocre.imagio.leeImagen: %s no esta en 'pgm' ni 'pbm'\n", nf);
      exit (10);
      break ;
    }
}

void escribeCabeceraImagen1 (FILE *fd, timagf imagf)
{
  int n, nc;
  fprintf (fd, "P5\n");
  nc = imagf->ncom;
  if (nc>0) {
    for (n=0; n<nc; n++)
      fprintf (fd, "%s", imagf->comen[n]);
    }
}

void escribeCabeceraImagen2v (FILE *fd, tvent vent)
{
  fprintf (fd, "%1d %1d\n", vent->tx, vent->ty);
  fprintf (fd, "%1d\n", vent->imaBase->ngr - 1);
}

void escribeCabeceraImagenMascomentarios (FILE *fd, char ** comen2)
{
  char ** c2;
  c2 = comen2;
  if (c2 != (char **)0L)
    while (*c2 != (char *) 0L) {
      fprintf (fd, "# %s\n", c2[0]);
      c2 = &(c2[1]);
      }
}

void escribeVentanaMascomentariosRect (tvent vent, char * nf, char ** comen2)
{
  FILE * fd;
  int y, tx, ty;
  char ** pix;
  fd = fopen (nf, "w");
  compruebaFd (fd, nf, "imagio.escribeVentanaMascomentarios");
  escribeCabeceraImagen1 (fd, vent->imaBase);
  escribeCabeceraImagenMascomentarios (fd, comen2);
  escribeCabeceraImagen2v (fd, vent);
  tx  = vent->tx;
  ty  = vent->ty;
  pix = vent->pix;
  for (y=0; y<ty; y++)
    fwrite (pix[y], 1, tx, fd);
        /* if mal fwrite */
  fclose (fd);
      /* if traza ... */
}

void escribeVentanaMascomentariosIncl (tvent vent, char * nf, char ** comen2)
{
  int fondo;
  tvent ventl;
//  tdcaracHistoGr carHisGr;
/*
  carHisGr = carhisgrDeVentana (vent);
  fondo = carHisGr->xblanco ;
  fondo = 192;
  fondo = fondoClaroFijo (vent);
*/
  fondo = fondoClaro (vent);
  ventanaLimpDeVentanaIncl (&ventl, vent, fondo);
  escribeVentanaMascomentariosRect (ventl, nf, comen2);
}

void escribeVentanaMascomentarios (tvent vent, char * nf, char ** comen2)
{
  if (vent->incl == 0)
    escribeVentanaMascomentariosRect (vent, nf, comen2);
  else
    escribeVentanaMascomentariosIncl (vent, nf, comen2);
}

void escribeVentanaLimMascomentarios (tvent vent, char * nf, char ** comen2)
{
  tvent vente;
  if (vent->venlimpia != 0L)
    vente = vent->venlimpia ;
  else
    vente = vent ;
//  vente = vent ;
  escribeVentanaMascomentarios (vente, nf, comen2);
}

void escribeVentana (tvent vent, char * nf)
{
  escribeVentanaMascomentarios (vent, nf, (char **) 0L );
}

void escribeImagen (timagf imagf, char * nf)
{
  escribeVentana (imagf->ven0, nf);
}

==> ocre_v0_033.tgz: ocre-0.033/ocre/src/imagio.c <==

#include <libintl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define _(mens) gettext(mens)
#define N_(mens) (mens)

#include "config.h"
#include "errors.h"
#include "imag.h"

enum {tfimagDesconocida, tfimagPbm, tfimagPgm, tfimagPpm, tfimagPbmAsc, tfimagPgmAsc};

FILE * mifopen (char *nf, char *modo)
{
  FILE * fd;
  char nfl[1024];
  fd = fopen (nf, modo);
  if (fd != 0L)
     return fd;
  strcpy  (nfl, "dat");
  strncat (nfl, "/", 2);
  strncat (nfl, nf, 128);
  fd = fopen (nfl, modo);
  return fd;
}

int tipofImagen (char * nf)
{
  FILE * fd;
  char  buf[MAXCOM] ;
  fd = mifopen (nf, "r");
  compruebaFd (fd, nf, "imagio.tipofImagen");
  fgets (buf, MAXCOM, fd);
  fclose (fd);
//
  if (strcmp(buf, "P1\n")==0)
     return tfimagPbmAsc ;
  if (strcmp(buf, "P2\n")==0)
     return tfimagPgmAsc ;
  if (strcmp(buf, "P4\n")==0)
     return tfimagPbm ;
  if (strcmp(buf, "P5\n")==0)
     return tfimagPgm ;
  if (strcmp(buf, "P6\n")==0)
     return tfimagPpm ;
  return tfimagDesconocida;
}

void leeCabeceraImagenPgbm (FILE *fd, timagf imagf, int tfima)
{
  char  buf[MAXCOM] ;
  char * vcom[MAXNLCOM];
  int n, nc;
  int tx, ty;
  int ng;
  fgets (buf, MAXCOM, fd);
      // asert P5
  nc = 0;
  do {
     fgets (buf, MAXCOM, fd);
     if (nc < MAXNLCOM) {
        vcom [nc] = strdup (buf);
        nc ++;
        }
     }
  while (buf[0] == '#');
  nc --;
  free (vcom[nc]);
  while (buf[0] == '\n')
     fgets (buf, MAXCOM, fd);
  imagf->comen = (char **) malloc (nc * sizeof(char *));
  for(n=0; n<nc; n++)
     imagf->comen[n] = vcom[n];
  imagf->ncom = nc;
  sscanf (buf, "%d %d", &tx, &ty);
  if (tx > MAXPIXH) {
     printf (_("%s: x_dimension (%4d) larger than anticipated (%4d)\n"),
               "ocre.imagio.leeCabeceraImagenPgbm", tx, MAXPIXH);
     exit (9);
     }
  if (ty > MAXPIXV) {
     printf (_("%s: y_dimension (%4d) larger than anticipated (%4d)\n"),
               "ocre.imagio.leeCabeceraImagenPgbm", ty, MAXPIXV);
     exit (9);
     }
  imagf->tx   = tx;
  imagf->ty   = ty;
  if (tfima == tfimagPgm) {
     fscanf (fd, "%d", &ng);
         // asert ng == 255
     fread (buf, 1, 1, fd);   // lee el cambio de linea 
     }
}

void leePixelsPbm (char *donde, int tx, int ty, FILE * fd)
{
  int x, y;
  char c;
  int m = 128;   /* '= 128' para no tener avisos(warnings) */
  char *d;
  d = donde;
  for (y=0; y<ty; y++){
     for (x=0; x<tx; x++){
        if ((x%8)==0) {
           fread (&c, 1, 1, fd);
           if (c <0) c = c + 256;
           m = 128;
           }
        if (c & m)
           *d =  32;
        else
           *d = 224;
        m /= 2;
        d++ ;
        }
     }
}

void leeImagenPgbm (timagf * nimagf, char * nf, int tfima)
{
  FILE * fd;
  timagf imagf;
  int tam;
  fd = mifopen (nf, "r");
  compruebaFd (fd, nf, "imagio.leeImagenPgm");
  imagf = imagfalloc ();
  leeCabeceraImagenPgbm (fd, imagf, tfima);
  tam  = imagf->tx * imagf->ty;
  imagf->apixels = (char *) malloc ((size_t)tam) ;
      // if mal malloc 
  if (tfima == tfimagPgm) {
     fread (imagf->apixels, 1, tam, fd);
     // if mal fread 
     }
  else /* tfima == tfimaPbm */ {
     leePixelsPbm (imagf->apixels, imagf->tx, imagf->ty, fd);
     }
  fclose (fd);
  imagf->nf = strdup (nf);
      // if traza ...
  imagf->ven0 = ventBaseDeImagf (imagf);
  * nimagf = imagf;
}

void leeImagen (timagf * nimagf, char * nf)
{
  switch (tipofImagen(nf)) {
     case tfimagPbm :
        leeImagenPgbm (nimagf, nf, tfimagPbm);
        break ;
     case tfimagPgm :
        leeImagenPgbm (nimagf, nf, tfimagPgm);
        break ;
     case tfimagPbmAsc :
     case tfimagPgmAsc :
     case tfimagPpm :
     case tfimagDesconocida :
        printf ("ocre.imagio.leeImagen: %s no esta en 'pgm' ni 'pbm'\n", nf);
        exit (10);
        break ;
     }
}

void escribeCabeceraImagen1 (FILE *fd, timagf imagf)
{
  int n, nc;
  fprintf (fd, "P5\n");
  nc = imagf->ncom;
  for (n=0; n<nc; n++)
     fprintf (fd, "%s", imagf->comen[n]);
}

void escribeCabeceraImagen2v (FILE *fd, tvent vent)
{
  fprintf (fd, "%1d %1d\n", vent->tx, vent->ty);
  fprintf (fd, "%1d\n", vent->imaBase->ngr - 1);
}

void escribeCabeceraImagenMascomentarios (FILE *fd, char ** comen2)
{
  char ** c2;
  c2 = comen2;
  if (c2 != (char **) 0L)
     while (*c2 != (char *) 0L) {
        fprintf (fd, "# %s\n", c2[0]);
        c2 = &(c2[1]);
        }
}

void escribeVentanaMascomentariosRect (tvent vent, char * nf, char ** comen2)
{
  FILE * fd;
  int y, tx, ty;
  char ** pix;
  fd = fopen (nf, "w");
  compruebaFd (fd, nf, "imagio.escribeVentanaMascomentarios");
  escribeCabeceraImagen1 (fd, vent->imaBase);
  escribeCabeceraImagenMascomentarios (fd, comen2);
  escribeCabeceraImagen2v (fd, vent);
  tx  = vent->tx;
  ty  = vent->ty;
  pix = vent->pix;
  for (y=0; y<ty; y++)
     fwrite (pix[y], 1, tx, fd);
        // if mal fwrite
  fclose (fd);
      // if traza ... 
}

void escribeVentanaMascomentariosIncl (tvent vent, char * nf, char ** comen2)
{
  int fondo;
  tvent ventl;
  fondo = fondoClaro (vent);
  ventanaLimpDeVentanaIncl (&ventl, vent, fondo);
  escribeVentanaMascomentariosRect (ventl, nf, comen2);
}

void escribeVentanaMascomentarios (tvent vent, char * nf, char ** comen2)
{
  if (vent->incl == 0)
     escribeVentanaMascomentariosRect (vent, nf, comen2);
  else
     escribeVentanaMascomentariosIncl (vent, nf, comen2);
}

void escribeVentanaLimMascomentarios (tvent vent, char * nf, char ** comen2)
{
  tvent vente;
  if (vent->venlimpia != 0L)
     vente = vent->venlimpia ;
  else
     vente = vent ;
  escribeVentanaMascomentarios (vente, nf, comen2);
}

void escribeVentana (tvent vent, char * nf)
{
  escribeVentanaMascomentarios (vent, nf, (char **) 0L );
}

void escribeImagen (timagf imagf, char * nf)
{
  escribeVentana (imagf->ven0, nf);
}
